package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CRFServiceClient;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StandConvertCrfBatchDTO;
import com.diagbot.dto.StandConvertCrfDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.entity.TestwordInfo;
import com.diagbot.entity.TestwordRes;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.model.ai.AIAnalyze;
import com.diagbot.service.TestwordInfoService;
import com.diagbot.service.TestwordResService;
import com.diagbot.service.impl.TestwordInfoServiceImpl;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.SearchData;
import com.diagbot.vo.StandConvertCrfVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhoutg
 * @time: 2020/7/29 15:03
 */
@Component
public class TestwordInfoFacade extends TestwordInfoServiceImpl {

    @Autowired
    CRFServiceClient crfServiceClient;
    @Autowired
    TestwordResFacade testwordResFacade;
    @Autowired
    @Qualifier("testwordResServiceImpl")
    TestwordResService testwordResService;
    @Autowired
    @Qualifier("testwordInfoServiceImpl")
    TestwordInfoService testwordInfoService;
    @Autowired
    TestFacade testFacade;
    @Autowired
    RetrievalFacade retrievalFacade;

    public Map getWord() {
        Map<String, String> res = new LinkedHashMap<>();
        Long start = System.currentTimeMillis();

        AIAnalyze aiAnalyze = new AIAnalyze(crfServiceClient);

        // 症状标准词
        List<String> symptomStand = getStandWord(9, 10000).getSymptomNames();
        // 诊断标准词
        List<String> diseaseStand = getStandWord(4, 100000).getDiseaseNames().stream().map(r -> r.getName()).collect(Collectors.toList());
        // 药品标准词
        List<String> drugStand = getStandWord(5, 100000).getDrugNames().stream().map(r -> r.getName()).collect(Collectors.toList());
        // 手术标准词
        List<String> operateStand = getStandWord(6, 100000).getOperationNames().stream().map(r -> r.getName()).collect(Collectors.toList());

        // 模型处理数据
        WordCrfDTO wordCrfDTO = new WordCrfDTO();
        List<TestwordInfo> testwordInfoList = this.page(new Page<>(1, 100), new QueryWrapper<TestwordInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("is_deal", 0)
                .orderByAsc("id")).getRecords();

        while (ListUtil.isNotEmpty(testwordInfoList)) {
            List<TestwordRes> insertRes = new ArrayList<>(); // 结果集

            Set<String> symptomList = new LinkedHashSet<>();
            Set<String> diseaseList = new LinkedHashSet<>();
            Set<String> drugList = new LinkedHashSet<>();
            Set<String> operateList = new LinkedHashSet<>();
            Set<String> lisList = new LinkedHashSet<>();
            Set<String> pacsList = new LinkedHashSet<>();
            for (TestwordInfo bean : testwordInfoList) {
                String text = bean.getText();
                String type = bean.getType();
                if ("主诉".equals(type) || "现病史".equals(type)) {
                    SearchData searchData = new SearchData();
                    searchData.setSymptom(text);
                    aiAnalyze.aiProcess(searchData, wordCrfDTO);
                    // 临床表现
                    CoreUtil.addSet(symptomList, CoreUtil.getName(wordCrfDTO.getPresentLabel().getClinicals()));
                    // 诊断
                    CoreUtil.addSet(diseaseList, CoreUtil.getName(wordCrfDTO.getPresentLabel().getDiags()));
                    // 药品
                    CoreUtil.addSet(drugList, CoreUtil.getName(wordCrfDTO.getPresentLabel().getMedicines()));
                    // 手术
                    CoreUtil.addSet(operateList, CoreUtil.getName(wordCrfDTO.getPresentLabel().getOperations()));
                    // // 化验
                    // CoreUtil.addSet(drugList, CoreUtil.getName(wordCrfDTO.getPresentLabel().getDiags()));
                    // // 辅检
                    // CoreUtil.addSet(diseaseList, CoreUtil.getName(wordCrfDTO.getPresentLabel().getDiags()));
                }
            }

            // 处理症状
            dealBusiness(symptomList, "症状", symptomStand, insertRes);
            // 处理诊断
            dealBusiness(diseaseList, "诊断", diseaseStand, insertRes);
            // 处理药品
            dealBusiness(drugList, "药品", drugStand, insertRes);
            // 处理手术
            dealBusiness(operateList, "手术", operateStand, insertRes);

            // 保存结果
            if (ListUtil.isNotEmpty(insertRes)) {
                testwordResService.saveBatch(insertRes);
            }
            // 更新处理标记
            for (TestwordInfo testwordInfo : testwordInfoList) {
                testwordInfo.setIsDeal(1);
            }
            testwordInfoService.saveOrUpdateBatch(testwordInfoList);

            // 获取下一批数据
            testwordInfoList = this.page(new Page<>(1, 100), new QueryWrapper<TestwordInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("is_deal", 0)
                    .orderByAsc("id")).getRecords();
        }

        long end = System.currentTimeMillis();
        res.put("执行时间", (end - start) / 1000.0 + "秒");
        return res;
    }


    public RetrievalDTO getStandWord(Integer type, Integer size) {
        RetrievalVO retrievalVO = new RetrievalVO();
        retrievalVO.setType(type);
        retrievalVO.setInputStr("");
        retrievalVO.setSize(size);
        return retrievalFacade.index(retrievalVO);
    }

    /**
     * 处理业务
     * @param list
     * @param type
     * @param standList
     */
    public void dealBusiness(Set<String> list, String type, List<String> standList, List<TestwordRes> insertRes) {
         if (list != null && list.size() > 0) {
            List<TestwordRes> testwordResList = testwordResFacade.list(new QueryWrapper<TestwordRes>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("word", list)
                    .eq("type", type)
            );
            // 剔除数据库已存在
            removeWord(list, testwordResList);
            // 剔除标准
            list.removeAll(standList);
            // 转换成实体类
            List<TestwordRes> insert = stringConvertRes(list, type);
            // 保存
            if (ListUtil.isNotEmpty(insert)) {
                insertRes.addAll(insert);
            }
        }
    }

    /**
     * string转实体类
     * @param stringList
     * @param type
     * @return
     */
    public List<TestwordRes> stringConvertRes(Set<String> stringList, String type) {
        List<TestwordRes> res = new ArrayList<>();
        for (String s : stringList) {
            TestwordRes testwordRes = new TestwordRes();
            testwordRes.setWord(s);
            testwordRes.setType(type);
            res.add(testwordRes);
        }
        return res;
    }

    /**
     * 剔除数据库已存在的数据
     * @param stringList
     * @param testwordResList
     */
    public void removeWord(Set<String> stringList, List<TestwordRes> testwordResList) {
        if (ListUtil.isEmpty(testwordResList)) {
            return ;
        }
        Set<String> setList = testwordResList.stream().map(r -> r.getWord()).collect(Collectors.toSet());
        stringList.removeAll(setList);
    }

    public void wordConvert() {
        List<TestwordRes> testwordInfoList = testwordResService.page(new Page<>(1, 20), new QueryWrapper<TestwordRes>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("is_deal", 0)
                .orderByAsc("id")).getRecords();

        while(ListUtil.isNotEmpty(testwordInfoList)) {
            List<StandConvertCrfVO> convertCrfVOList = new ArrayList<>();
            for (TestwordRes testwordRes : testwordInfoList) {
                StandConvertCrfVO standConvertCrfVO = new StandConvertCrfVO();
                standConvertCrfVO.setWord(testwordRes.getWord());
                switch (testwordRes.getType()) {
                    case "症状":
                        standConvertCrfVO.setWord_type("symptom");
                        standConvertCrfVO.setNumber(3);
                        convertCrfVOList.add(standConvertCrfVO);
                        break;
                    default:
                        break;
                }
            }
            StandConvertCrfBatchDTO standConvertCrfBatchDTO = testFacade.testStandConvertBatch(convertCrfVOList);
            for (TestwordRes testwordRes : testwordInfoList) {
                Set<String> symptomType = new LinkedHashSet<>();
                switch (testwordRes.getType()) {
                    case "症状":
                        Map<String, StandConvertCrfDTO> convertCrfDTOMap = standConvertCrfBatchDTO.getData().get("symptom");
                        if (convertCrfDTOMap != null && convertCrfDTOMap.get(testwordRes.getWord()) != null) {
                            List<Map<String,String>> listMap = convertCrfDTOMap.get(testwordRes.getWord()).getStandard_words();
                            int size = 0;
                            for (Map<String,String> map : listMap) {
                                if (size < 3 && StringUtil.isNotBlank(map.get("standard_word"))) {
                                    symptomType.add(map.get("standard_word"));
                                }
                                size++;
                            }
                            testwordRes.setConvertWord(StringUtils.join(symptomType, "；"));
                        }
                        break;
                    default:
                        break;
                }
                testwordRes.setIsDeal(1);
            }
            testwordResService.saveOrUpdateBatch(testwordInfoList);

            testwordInfoList = testwordResService.page(new Page<>(1, 20), new QueryWrapper<TestwordRes>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("is_deal", 0)
                    .orderByAsc("id")).getRecords();
        }
    }
}
